/*
 * si4133_tiny13a.c
 *
 * Created: 27.10.2016 15:34:53
 * Author : Yevhen Ulianov <cd4011a@gmail.com>
 */ 


// FUSES
// low: 79
// high: FF
// INTRC Osc. 4.8MHz 64ms

#define F_CPU 4800000L

#include <avr/io.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>


// список задаваемых значений NRF2
#define SI_NRF2_1 1152
#define SI_NRF2_2 1151
#define SI_NRF2_3 1296
#define SI_NRF2_4 1297

// RF2 R-Divider (установка F reference)
#define SI_RRF2 10

// Назначение выводов контроллера для управления синтезатором
#define SI_PORT PORTB
#define SI_DDR DDRB
#define SI_DATA PB1
#define SI_CLK PB0
#define SI_EN PB2

// описание перемычек, задающих частоту
#define KEY1 PINB3
#define KEY2 PINB4
#define KEY_PORT PORTB
#define KEY_DDR DDRB
#define KEY_PIN PINB


// Базовая конфигурация работы микросхемы синтезатора 

// Main configuration
#define SI_AUXSEL 0b11
#define SI_IFDIV 0
#define SI_LPWR 0
#define SI_AUTOPDB 0
#define SI_AUTOKP 1
#define SI_RFPWR 0

// Phase detector gain
#define SI_KPI 0
#define SI_KP1 0
#define SI_KP2 0

// Powerdown
#define SI_PDIB 0
#define SI_PDRB 1

// RF1 N-Divider
#define SI_NRF1 0

// RF2 N-Divider
#define SI_NRF2 1152

// IF N-Divider
#define SI_NIF 0

// RF1 R-Divider
#define SI_RRF1 0

// IF R-Divider
#define SI_RIF 0

#define NRF2_REG_ADDR 4


long unsigned si_nrf2;


void si_init(void) {
	SI_PORT &= ~(1<<SI_CLK) & ~(1<<SI_DATA);					// SI_CLK = 0; SI_DATA =0;
	SI_PORT |= (1<<SI_EN);										// SI_EN = 1;
	SI_DDR |= (1<<SI_DATA) | (1<<SI_CLK) | (1<<SI_EN);
}


// функция передачи битовых последовательностей в микросхему синтезатора
// в формате, описанном в datasheet
// |D17|...|D0|A3|...|A0| - 22 бита
void si_send_raw (long unsigned data) {
	char i;
	
	data = data << 10;
	SI_PORT &= ~(1<<SI_EN);
	for (i=0;i<22;i++) {
		if (data & 0x80000000)
		{
			SI_PORT |= (1<<SI_DATA);
			} else {
			SI_PORT &= ~(1<<SI_DATA);
		}
		_delay_ms(1);
		SI_PORT |= (1<<SI_CLK);
		_delay_ms(1);
		SI_PORT &= ~(1<<SI_CLK);
		_delay_ms(1);
		data = data << 1;
	}
	SI_PORT |= (1<<SI_EN);
}



// возвращает сформированное значение конфигурационного регистра
// исходя из значений описанных в #define и переменных nrf2 и rrf2
long unsigned si_reg_data(char addr) {
	long unsigned data;
	
	data = 0;
	switch (addr) {
		case 0 :
			data |=  SI_AUXSEL & 0b11 ;
			data = data << 2;
			data |= SI_IFDIV & 0b11;
			data = data << 5;
			data |= SI_LPWR & 1;
			data = data << 2;
			data |= SI_AUTOPDB & 1;
			data = data << 1;
			data |= SI_AUTOKP & 1;
			data = data << 1;
			data |= SI_RFPWR & 1;
			data = data << 1;
			break;
		case 1 :
			data |=  SI_KPI & 0b11 ;
			data = data << 2;
			data |=  SI_KP2 & 0b11 ;
			data = data << 2;
			data |=  SI_KP1 & 0b11 ;
			break;
		case 2 :
			data |=  SI_PDIB & 1 ;
			data = data << 1;
			data |=  SI_PDRB & 1 ;
			break;
		case 3 :
			data |= SI_NRF1 & 0b111111111111111111 ;
			break;
		case 4 :
			data |= si_nrf2 & 0b11111111111111111 ;
			break;
		case 5 :
			data |= SI_NIF & 0b11111111111111111 ;
			break;
		case 6 :
			data |= SI_RRF1 & 0b1111111111111 ;
			break;
		case 7 :
			data |= SI_RRF2 & 0b1111111111111 ;
			break;
		case 8 :
			data |= SI_RIF & 0b1111111111111 ;
			break;
		default:
			break;
	}
	return data;
}


// загружает в микросхему синтезатора значение регистра с 
// указанным адресом addr
void si_set_reg(char addr) {
	long unsigned data;
	
	data = si_reg_data(addr);
	data = data << 4;
	data |= addr;				// добавление адреса регистра в последовательность бит
	si_send_raw(data);
}



// загрузка значений всех регистров в микросхему синтезатора
void si_set(void) {
	long unsigned data;
	char i;
	
	for (i=0;i<9;i++)
	{
		data = si_reg_data(i);
		data = data << 4;
		data |= i;				// добавление адреса регистра в последовательность бит
		si_send_raw(data);
	}
}




int main(void)
{
	
	KEY_DDR &= ~(1<<KEY1)&~(1<<KEY2);
	KEY_PORT |= (1<<KEY1)|(1<<KEY2);		// включаем подтягивающие резисторы
	
	//_delay_ms(500);
	si_init();	
	
	// проверяем не замкнуты ли кнопки между собой (режим проверки граничных значений частоты)
	if(!(~KEY_PIN & (1<<KEY1)) & !(~KEY_PIN & (1<<KEY2))) {
		KEY_DDR |= (1<<KEY1);
		KEY_PORT &= ~(1<<KEY1);
		_delay_ms(1);
		if (~KEY_PIN & (1<<KEY2)) {
			si_set();
			while (1)
			{
				si_nrf2 = 0;
				si_set_reg(NRF2_REG_ADDR);
				_delay_ms(3000);
				si_nrf2 = 0xFF;
				si_set_reg(NRF2_REG_ADDR);
				_delay_ms(3000);
			}
		} else {
			KEY_DDR &= ~(1<<KEY1);
			KEY_PORT |= (1<<KEY1);
		}
	}
	
	
	// устанавливаем выставленную частоту
	
	si_nrf2 = SI_NRF2_1;			// не нажаты кнопки 
	if (~KEY_PIN & (1<<KEY1)){
		si_nrf2 = SI_NRF2_2;		// нажата KEY1
		if (~KEY_PIN & (1<<KEY2)){
			si_nrf2 = SI_NRF2_4;	// нажаты KEY1 и KEY2
		}
	} else {
		if (~KEY_PIN & (1<<KEY2)){	// нажата только KEY2
			si_nrf2 = SI_NRF2_3;
		}
	};
	si_set();


	// power down
	cli();							// отключаем прерывания, чтобы не проснуться :)
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);	
	while (1)
	{
		sleep_enable();
		sleep_cpu(); 
	}
	
}

